module Rack

  module ConsoleAuth
    class Authorization
      require "#{::File.dirname(__FILE__)}/console_auth"

      def initialize(app)
        @app = app
      end

      def unauthorized(env, params = Hash.new)
        header  = (params[:header]  || 'Unauthorized')
        message = (params[:message] || 'You are not authorized to access this page')
        status  = (params[:status]  || 401)

        [ status, {'Content-Type' => 'text/html'}, ERB.new(IO.read("#{::File.dirname(__FILE__)}/../views/unauthorized.erb")).result(binding) ]
      end

      def call(env)
        path_url = env['REQUEST_URI']

        if env.has_key? 'rack.cas.client.user' or env['rack.session'].has_key? 'cas' or env['rack.session'].has_key? 'rack.basic.user'
          begin
            user = ::ConsoleAuth.get_user(env)
          rescue
            return [ 301, {'Location' => '/logout'}, 'Corrupt session. Clearing']
          end

          if user
            role = user.roles.first.name
            env['rack.session']['CONSOLE_AUTH_ACCOUNT_NAME'] = user.username
            env['rack.session']['CONSOLE_AUTH_ACCOUNT_ROLE'] = role
            env['rack.session']['CONSOLE_AUTH_AUTHENTICATOR'] = user.origin
          else
            msg = <<-MESSAGE
              Information matching the user credentials could not be found in the database. This could be because
              this user's account has been deleted, or may be because of database connectivity issues. Contact your
              administrator if this problem persists.
              <br><p><a href="/logout">Log out and try again</a></p>
            MESSAGE

            return unauthorized env, :header => 'User could not be retrieved', :message => msg, :status => 410
          end
        else
          user = nil
          role = ::ConsoleAuth.allow_global_unauthenticated_access? ? 'read-only' : nil
        end

        if user and user.status != 'enabled'
          header, message = case user.status
                            when 'disabled'
                              ['Account Disabled', 'Your account is currently disabled. Please contact your systems administrator.']
                            when 'pending'
                              ['Account Awaiting Activation', 'Your account is currently pending.  Please check your email for an activation link.']
                            else
                              [nil, nil]
                            end

          return unauthorized env, :header => header, :message => message
        end

        roles = case role
               when 'read-write'
                 [:read, :write]
               when 'read-only'
                 [:read]
               when 'admin'
                 [:read, :write, :admin]
               when nil
                 []
               end

        log_user_id, log_user_name  = user.nil? ? [ String.new, 'anonymous'] : [user.id, user.username]

        begin
          request_method = env['REQUEST_METHOD'].downcase.to_sym
          access_control_role = ::ConsoleAuth.access_control_role(:url    => path_url,
                                                          :roles  => roles,
                                                          :method => request_method)

          restriction = ::ConsoleAuth.find_restriction :url => path_url
          if restriction and restriction[request_method].roles.include? :write
            ::ConsoleAuth.logger.info("User #{log_user_id} (#{log_user_name}) accessed read-write url #{path_url}")
          end
        rescue ::ConsoleAuth::Unauthorized
          ::ConsoleAuth.logger.info("User #{log_user_id} (#{log_user_name}) attempted to access unauthorized URL #{path_url}")
          return unauthorized env
        end

        env['rack.session']['ACCESS_CONTROL_ROLE'] = access_control_role
        @app.call(env)
      end

    end
  end
end
